﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
// 추가
using MyOpenBom.ItemMan;
using MyOpenBom.Common;
using System.Data.SqlClient;
using FxLib.Utils;

namespace MyOpenBom.BomMan
{
    public partial class frmBOMMan : Form
    {
        clsItem oItem;
        clsBOMManagement oBom;      // T_BOM 테이블을 사용하는 객체

        string p_code;              // 트리의 부모 품목코드
        string p_name;              // 트리의 부모 품목명

        public frmBOMMan()
        {
            InitializeComponent();

            // 품목코드 객체 초기화
            oItem = new clsItem();

            // 모품목코드 조회 이력 컨트롤 초기화
            lstPCodeHistory.Items.Clear();
            lstPCodeHistory.Columns.Add("품목코드", 70, HorizontalAlignment.Left);
            lstPCodeHistory.Columns.Add("품목명", 100, HorizontalAlignment.Left);
        }

        /// <summary>
        /// 모품목코드 조회 기능
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtParentItemCode_KeyPress(object sender, KeyPressEventArgs e)
        {
            // 엔터키 또는 탭키를 입력받으면 해당 코드에 대한 모품목 코드를 조회할 수 있도록 한다.
            if (e.KeyChar == 13)
            {
                Form oBom = new frmItemParentList(txtParentItemCode.Text, oItem);
                oBom.ShowDialog();
                // 모품목코드 선택 대화창에서 사용자가 선택한 정보를 텍스트 컨트롤에 설정한다.
                txtParentItemCode.Text = p_code = oItem.Code;
                txtParentItemName.Text = p_name = oItem.Im_desc;
            }
        }

        private void txtParentItemCode_TextChanged(object sender, EventArgs e)
        {

        }

        private void LoadChildItem(string p_code)
        {
            // BOM 객체의 코드 조회 기능이 여러번 수행될 수 있다.
            // 따라서 해당 객체가 이미 생성되었는지 체크한다.
            if (oBom == null)
                oBom = new clsBOMManagement(p_code);
            else
                oBom.GetRoot(p_code);

            dgChild.DataSource = oBom.GetDataSet();
            dgChild.DataMember = "t_bom";

            dgChild.Columns[0].HeaderText = "품목코드";
            dgChild.Columns[1].HeaderText = "품목명";
            dgChild.Columns[1].Width = 190;
            dgChild.Columns[2].HeaderText = "품목구분";
            dgChild.Columns[3].HeaderText = "필요수량";
            dgChild.Columns[4].HeaderText = "공급일수";
        }

        private void updateTreeBom(string p_code, string p_name)
        {
            treeBOM.BeginUpdate();
            treeBOM.Nodes.Clear();
            treeBOM.Nodes.Add(p_code + "-" + p_name);

            //// 품목 구분정보를 콤보박스에 설정한다.
            //foreach (CodeComponent oCode in oBom.GetNodes())
            //    treeBOM.Nodes[0].Nodes.Add(oCode.Code() + "-" + oCode.Name());

            // Element 단위로 Composite 패턴에 추가한다.
            DataSet oDS = oBom.GetDataSet();
            foreach (DataRow dr in oDS.Tables[0].Rows)
            {
                treeBOM.Nodes[0].Nodes.Add(dr["c_code"].ToString().Trim() + "-" +
                    dr["im_desc"].ToString().Trim() + "(" + dr["qty"].ToString() + ")");
            }
            treeBOM.EndUpdate();
        }

        /// <summary>
        /// 사용자가 선택한 모품목코드 이력관리 리스트뷰 컨트롤에
        /// 이미 등록된 값이 있는지 체크한다. 중복 입력 체크
        /// </summary>
        /// <param name="strCode"></param>
        /// <returns>해당 코드가 없으면 false를 반환한다.</returns>
        private bool CheckParentCodeHistory(string strCode)
        {
            if (lstPCodeHistory.FindItemWithText(strCode) == null)
                return false;

            return true;
        }

        private void btnChildQuery_Click(object sender, EventArgs e)
        {
            if (txtParentItemName.Text == "")
            {
                MessageBox.Show("먼저 모품목코드를 조회하세요.");
                return;
            }

            MessageBox.Show("모품목코드: " + txtParentItemCode.Text + " 조회합니다.");
            LoadChildItem(txtParentItemCode.Text);
            updateTreeBom(txtParentItemCode.Text, txtParentItemName.Text);

            // 현재 선택된 모품목 정보를 저장한다.
            p_code = txtParentItemCode.Text;
            p_name = txtParentItemName.Text;

            // 사용자가 조회한 모품목코드 정보를 이력관리한다.
            if (!CheckParentCodeHistory(p_code))
            {
                ListViewItem item;
                item = new ListViewItem(p_code);
                item.SubItems.Add(p_name);
                lstPCodeHistory.Items.Add(item);
            }

            // 현재 선택된 모품목 정보를 보여준다.
            lblPCode.Text = p_code + "-" + p_name;

            // 추가/삭제 버튼을 활성화한다.
            btnChildAdd.Enabled = true;
            btnChildDelete.Enabled = true;
        }

        private void btnChildAdd_Click(object sender, EventArgs e)
        {
            try
            {
                Form oChild = new frmItemChildList(oItem);
                oChild.ShowDialog();

                MessageBox.Show("모품목코드: " + p_code + ", 자품목코드: " + oItem.Code +
                    " 를 추가합니다.");

                // 모품목코드와 자품목코드가 동일한 값이 BOM에 입력될 수 없다.
                if (p_code.Equals(oItem.Code))
                {
                    MessageBox.Show("모품목과 동일한 자품목 정보를 추가할 수 없습니다.");
                    return;
                }

                // TODO: 자품목코드 추가 시 자품목의 하위 품목코드가 모품목코드의 하위 자품목
                // 코드를 포함해서는 안된다.
                // 모품목코드, 자품목코드, 필요수량을 T_BOM 테이블에 입력한다.
                if (oBom.Create(p_code, oItem.Code, oItem.Ss) == false)
                {
                    MessageBox.Show(oItem.Code + "의 하위 품목으로 " + p_code + "가 존재합니다. \n" +
                        "추가될 수 없습니다.");
                }
                // 자품목코드 테이블을 리프레쉬한다.
                LoadChildItem(p_code);
                updateTreeBom(p_code, p_name);
            }
            catch (SqlException err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                if (err.Number == 2627)
                {
                    MessageBox.Show(MyOpenBom.Properties.Resources.strErr03, "알림창");
                }
                else
                    MessageBox.Show(MyOpenBom.Properties.Resources.strErr02 + err.Number);
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        private void btnChildDelete_Click(object sender, EventArgs e)
        {
            try
            {
                if (MessageBox.Show("모품목코드: " + p_code + ", " + 
                    dgChild.CurrentRow.Cells[1].Value.ToString() + 
                    " - 자품목정보를 정말 삭제하시겠습니까?",
                    "BOM 코드 정보", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    // 자품목 정보를 BOM 테이블에서 삭제한다.
                    oBom.Remove(p_code , dgChild.CurrentRow.Cells[0].Value.ToString());
                    // 자품목코드 테이블을 리프레쉬한다.
                    LoadChildItem(p_code);
                    updateTreeBom(p_code, p_name);
                }
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }

        }

        private void treeBOM_Click(object sender, EventArgs e)
        {
        }

        private void treeBOM_AfterSelect(object sender, TreeViewEventArgs e)
        {
            try
            {
                // 선택된 노드가 없으면 루틴 종료한다.
                if (treeBOM.SelectedNode == null)
                    return;

                // 코드 부분을 조회한다.
                string[] arCodeName = treeBOM.SelectedNode.Text.Split(new Char[] {'-', '(', ')'});
                //string[] arCodeName = CodeName(treeBOM.SelectedNode.Text);

                // 부모 품목코드 설정
                txtParentItemCode.Text = arCodeName[0];
                txtParentItemName.Text = arCodeName[1];

                // 만약 하위노드가 존재하면 루틴 종료한다.
                int myNodeCount = treeBOM.SelectedNode.GetNodeCount(true);
                if (myNodeCount > 0)
                    return;

                // 품목코드 정보 조회
                oBom.GetRoot(arCodeName[0]);
                DataSet oDS = oBom.GetDataSet();
                // Element 단위로 Composite 패턴에 추가한다.
                foreach (DataRow dr in oDS.Tables[0].Rows)
                {
                    treeBOM.SelectedNode.Nodes.Add(dr["c_code"].ToString().Trim() + "-" +
                        dr["im_desc"].ToString().Trim() + "(" + dr["qty"].ToString() + ")");
                }
                //// 품목 구분정보를 콤보박스에 설정한다.
                //foreach (CodeComponent oCode in oBom.GetNodes())
                //    treeBOM.SelectedNode.Nodes.Add(oCode.Code() + "-" + oCode.Name());

            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        private void lstPCodeHistory_MouseDoubleClick(object sender, MouseEventArgs e)
        {

        }

        /// <summary>
        /// 모품목 조회 이력창에서 사용자가 선택한 모품목코드를 표시한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lstPCodeHistory_SelectedIndexChanged(object sender, EventArgs e)
        {
            ListView.SelectedListViewItemCollection cItems = this.lstPCodeHistory.SelectedItems;

            foreach (ListViewItem item in cItems)
            {
                txtParentItemCode.Text = item.SubItems[0].Text;
                txtParentItemName.Text = item.SubItems[1].Text;
            }
        }

        private void txtClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
