﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
// 추가
using MyOpenBom.ItemMan;
using MyOpenBom.Common;
using System.Data.SqlClient;
using FxLib.Utils;
using System.IO;

namespace MyOpenBom.BomMan
{
    public partial class frmBOMTree : Form
    {
        clsItem oItem;
        clsBOMManagement oBom;      // T_BOM 테이블을 사용하는 객체

        int iFlag;                  // datagridview 의 표시 상태(0: 없음, 1: 품목전체, 2: 품목레벨);

        DataSet oDS;

        public frmBOMTree()
        {
            InitializeComponent();

            // 품목코드 객체 초기화
            oItem = new clsItem();
            iFlag = 0;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void txtItemCode_KeyPress(object sender, KeyPressEventArgs e)
        {
            // 엔터키 또는 탭키를 입력받으면 해당 코드에 대한 모품목 코드를 조회할 수 있도록 한다.
            if (e.KeyChar == 13)
            {
                Form oItemParentList = new frmItemParentList(txtItemCode.Text, oItem);
                oItemParentList.ShowDialog();
                // 모품목코드 선택 대화창에서 사용자가 선택한 정보를 텍스트 컨트롤에 설정한다.
                txtItemCode.Text = oItem.Code;
                txtItemName.Text = oItem.Im_desc;
            }
        }

        /// <summary>
        /// 마이너스 재고는 데이터그리드뷰 컨트롤의 특정행을 빨간색으로 표시한다.
        /// </summary>
        /// <param name="iRow">- 기준인 열번호</param>
        private void HighlightBomTree(int iColumn)
        {
            DataGridViewCellStyle highlightCellStyle = new DataGridViewCellStyle();
            highlightCellStyle.BackColor = Color.DeepPink;
            highlightCellStyle.Font = new Font(dvBOMTree.Font, dvBOMTree.Font.Style | FontStyle.Bold);

            foreach (DataGridViewRow r in dvBOMTree.Rows)
            {
                if (Convert.ToInt16(r.Cells[iColumn].Value) < 0)
                    r.DefaultCellStyle = highlightCellStyle;
            }
        }

        /// <summary>
        /// 품목별, 레벨, 재고, 재고부족분 등을 보여준다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnItemQuery_Click(object sender, EventArgs e)
        {
            try
            {
                iFlag = 1;      // 품목전체 부분

                if (Convert.ToInt16(txtLevel.Text) < 1 || Convert.ToInt16(txtLevel.Text) > 20)
                {
                    MessageBox.Show("1이상, 20이하여야 합니다.");
                }

                oBom = new clsBOMManagement(txtItemCode.Text);
                dvBOMTree.DataSource = oDS = oBom.QueryBOMNeedQty(txtItemCode.Text, txtItemQty.Text, txtLevel.Text);
                dvBOMTree.DataMember = "t_bom_tree";

                dvBOMTree.ColumnHeadersVisible = true;
                // "c": 한국 통화 표시
                // "#.#": 천단위 구분
                dvBOMTree.Columns[2].DefaultCellStyle.Format = "c";
                dvBOMTree.Columns[4].DefaultCellStyle.Format = "c";
                dvBOMTree.Columns[7].DefaultCellStyle.Format = "c";
                dvBOMTree.Columns[3].DefaultCellStyle.Format = "#,#";
                dvBOMTree.Columns[5].DefaultCellStyle.Format = "#,#";
                dvBOMTree.Columns[6].DefaultCellStyle.Format = "#,#";

                // 마이너스 재고를 빨간색으로 표시한다.
                HighlightBomTree(6);
            }
            catch (Exception err)
            {
                MessageBox.Show(err.ToString());
            }
        }

        private void btnTotalQuery_Click(object sender, EventArgs e)
        {
            try
            {
                iFlag = 2;      // 품목레벨

                oBom = new clsBOMManagement(txtItemCode.Text);
                dvBOMTree.DataSource = oDS = oBom.QueryBOMTree(txtItemCode.Text, txtItemQty.Text);
                dvBOMTree.DataMember = "t_bom_tree";

                dvBOMTree.ColumnHeadersVisible = false;
                dvBOMTree.Columns[0].Width = 600;
                dvBOMTree.Columns[0].HeaderText = "트리 구조";
            }
            catch (Exception err)
            {
                MessageBox.Show(err.ToString());
            }
        }

        /// <summary>
        /// 컬럼을 선택할때마다 정열이 발생한다.
        /// 마이너스 재고는 빨간줄로 표시한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dvBOMTree_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            // 품목전체표시일 경우만 datagridview의 (-) 재고 표시를 보여준다.
            if(iFlag == 1)
                HighlightBomTree(6);
        }

        /// <summary>
        /// datagridvieiw 컨트롤 내용을 xml 파일로 저장한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                // saveFileDialog()를 사용하면 사용자가 선택한 디렉토리가
                // 현재 디렉토리로 변경된다. 따라서 설정파일(config.xml) 파일을
                // 찾지 못하게 된다.
                string strCurrentDir = Directory.GetCurrentDirectory();

               // Displays a SaveFileDialog so the user can save the Image
               // assigned to Button2.
               SaveFileDialog saveFileDialog1 = new SaveFileDialog();
               saveFileDialog1.Filter = "엑셀파일|*.xls|모든 파일|*.*";
               saveFileDialog1.Title = "파일저장";
               saveFileDialog1.ShowDialog();

               // If the file name is not an empty string open it for saving.
               if (saveFileDialog1.FileName == "")
               {
                   return;
               }

                Cursor.Current = Cursors.WaitCursor;
                oDS.WriteXml(saveFileDialog1.FileName);
                MessageBox.Show("저장완료");
                Cursor.Current = Cursors.Default;

                Directory.SetCurrentDirectory(strCurrentDir);
            }
            catch (Exception err)
            {
                MessageBox.Show(err.ToString());
            }
        }

        /// <summary>
        /// 툴팁 적용
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_MouseHover(object sender, EventArgs e)
        {
            toolTip1.SetToolTip(btnSave, 
                "영역을 선택, Ctrl+C 복사해서 엑셀에 붙여넣기(유니코드로) 해보세요.");
        }
    }
}
