﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
// 추가
using MyOpenBom.ItemMan;
using FxLib.Utils;

namespace MyOpenBom.BomMan
{
    public partial class frmItemChildList : Form
    {
        clsItem oItem;

        public frmItemChildList(clsItem oItemTmp)
        {
            InitializeComponent();

            oItem = oItemTmp;
        }

        /// <summary>
        /// 자품목코드 정보를 조회한다.
        /// </summary>
        private void QueryChildItem()
        {
            try
            {
                DataSet dsCode = oItem.Read(txtCode.Text);

                lstItem.Items.Clear();
                ListViewItem item;

                lstItem.Columns.Add("품목코드", 70, HorizontalAlignment.Left);
                lstItem.Columns.Add("품목명", 150, HorizontalAlignment.Left);
                lstItem.Columns.Add("종류", 50, HorizontalAlignment.Left);

                foreach (DataRow r in dsCode.Tables["t_item"].Rows)
                {
                    item = new ListViewItem(r["code"].ToString());
                    item.SubItems.Add(r["im_desc"].ToString());
                    item.SubItems.Add(r["code_detail_nm"].ToString());
                    lstItem.Items.Add(item);
                }

            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }

        }

        /// <summary>
        /// 사용자가 선택한 자품목코드 정보를 목록에 표시한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCode_Click(object sender, EventArgs e)
        {
            try
            {
                QueryChildItem();
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        private void lstItem_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// 사용자가 선택한 자품목정보를 oitem을 통해서 부모 Form에 전달한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            try
            {
                if (txtQty.Text == "")
                {
                    MessageBox.Show("수량을 입력하세요");
                    return;
                }

                ListView.SelectedListViewItemCollection cItems = this.lstItem.SelectedItems;

                foreach (ListViewItem item in cItems)
                {
                    // 사용자가 품목개체의 품목코드와 품목명 정보를 객체에 설정한다.
                    // 아래 품목개체는 참조객체로 frmBOMMan 폼 에서 사용된다.
                    oItem.Code = item.SubItems[0].Text;
                    oItem.Im_desc = item.SubItems[1].Text;
                    oItem.Ss = txtQty.Text;                 // 원래 Ss 는 안전재고이지만 여기서는 
                                                            // 필요수량으로 사용한다.
                    this.Close();
                }

            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        /// <summary>
        /// 엔터키를 받으면 자품목코드 조회를 한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtCode_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
            {
                try
                {
                    QueryChildItem();
                }
                catch (Exception err)
                {
                    clsLog.Write(clsInfo.FATAL, err.ToString());
                    MessageBox.Show(err.ToString());
                }
            }
        }
    }
}
