﻿using System;
using System.Collections.Generic;
using System.Text;
// 추가
using System.Collections;
using System.Data;
using FxLib.Utils;
using System.Data.SqlClient;

namespace MyOpenBom.Common
{
    /// <summary>
    /// Template Method 패턴을 일부 응용한다.
    /// </summary>
    public class clsCategoryManagement : CodeManagement
    {
        public clsCategoryManagement(string code) : base(code)
        {

        }

        /// <summary>
        /// 특정 카테고리 마스터 정보의 하위 디테일 코드 정보를 조회한다.
        /// 해당 정보는 Code Composite 패턴을 통해서 관리된다.
        /// </summary>
        /// <param name="code"></param>
        public override void GetRoot(string code)
        {
            try
            {
                string strQuery = "select code_detail, code_detail_nm " + 
                    " from t_code_d " + 
                    " where code = '" + code + "'";

                clsDBHelper oDb = new clsDBHelper();
                // 거래처 정보를 저장한다.
                oDS = oDb.ExecuteQuery(strQuery, "t_code_d");
                oDb = null;

                // Element 단위로 Composite 패턴에 추가한다.
                foreach (DataRow dr in oDS.Tables[0].Rows)
                {
                    oCComposite.Add(new CodeElement(dr["code_detail"].ToString().Trim(),
                        dr["code_detail_nm"].ToString().Trim()));
                }

            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }

    }

    public class clsBOMManagement : CodeManagement
    {
        public clsBOMManagement(string code)
            : base(code)
        {

        }
        /// <summary>
        /// 특정 카테고리 마스터 정보의 하위 디테일 코드 정보를 조회한다.
        /// 해당 정보는 Code Composite 패턴을 통해서 관리된다.
        /// </summary>
        /// <param name="code"></param>
        public override void GetRoot(string code)
        {
            try
            {
                // 부모폼목코드(p_code) 에 대한
                // 자식품목코드(c_code), 품목명(im_desc), 품목구분(code_detail_nm), 
                // 필요수량(qty), 공급일수(리드타임) 조회
                string strQuery = "select bom.c_code, item.im_desc, code.code_detail_nm, bom.qty, item.sday  " + 
                    " from t_bom bom, t_item item, t_code_d code " +
                    " where bom.p_code = '" + code + "'" +
                    " and bom.c_code = item.code " + 
                    " and code.code = 'C0001' " +
                    " and item.cat = code.code_detail " + 
                    " and item.item_use = 'Y'";

                // TODO: 아래 코드 중복 해결해야 함
                clsDBHelper oDb = new clsDBHelper();
                // 거래처 정보를 저장한다.
                oDS = oDb.ExecuteQuery(strQuery, "t_bom");
                oDb = null;

                oCComposite.Clear();

                // Element 단위로 Composite 패턴에 추가한다.
                foreach (DataRow dr in oDS.Tables[0].Rows)
                {
                    oCComposite.Add(new CodeElement(dr["c_code"].ToString().Trim(),
                        dr["im_desc"].ToString().Trim()));
                }

            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }

        /// <summary>
        /// 모품목코드(p_code)와 자품목코드(c_code)간의 재귀적 루프 관계가 존재하는지 체크
        /// 1. c_code의 하위 자품목코드 중에 p_code가 존재하는지 체크한다. 
        ///     => SP_BOM_expand (@p_code varchar(6), @c_code varchar(6), @ret_code varchar(5) output)
        /// 2. 만약 존재하면 true를 반환하고, 재귀적 루프 관계가 될수 있음을 알린다.
        ///     만약 존재하지 않으면 false를 반환한다.
        /// </summary>
        /// <param name="p_code"></param>
        /// <param name="c_code"></param>
        /// <returns></returns>
        private bool CheckBOMTree(string p_code, string c_code)
        {
            try
            {
                clsDBHelper oDBHelper = new clsDBHelper();

                // c_code의 하위 자품목코드 중에 p_code가 존재하는지 체크한다. 
                string strQuery = "select sp_bom_expand('" + c_code + "', '" + p_code + "') ";

                SqlConnection dbConn = new SqlConnection(oDBHelper.getConnectionString());
		        dbConn.Open();
                oDBHelper = null;

		        SqlCommand cmd = new SqlCommand();
		        cmd.Connection = dbConn;
		        cmd.CommandType = CommandType.StoredProcedure;
		        cmd.CommandText = "dbo.SP_BOM_EXPAND";				
        		
		        cmd.Parameters.Add("@p_code", SqlDbType.VarChar, 6);
		        cmd.Parameters.Add("@c_code", SqlDbType.VarChar, 6);

		        cmd.Parameters["@p_code"].Value = c_code;
		        cmd.Parameters["@c_code"].Value = p_code;

		        SqlParameter myParameter = cmd.Parameters.Add("@ret_code", SqlDbType.VarChar, 5);
		        myParameter.Direction = ParameterDirection.Output;

		        cmd.ExecuteNonQuery();

		        string result_value = Convert.ToString(cmd.Parameters["@ret_code"].Value);

                if (result_value.Equals("TRUE"))
                    return true;
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }

            return false;
        }

        /// <summary>
        /// 품목코드의 특정 수량에 대한 
        /// 하위 전체 소요량을 표시한다.
        /// </summary>
        /// <returns></returns>
        public DataSet QueryBOMNeedQty(string code, string qty, string strLevel)
        {
            DataSet oDS = new DataSet();
            try
            {
                // 부모폼목코드(p_code) 에 대한
                // 자식품목코드(c_code), 품목명(im_desc), 품목구분(code_detail_nm), 
                // 필요수량(qty), 공급일수(리드타임) 조회
                string strQuery = "select a.code as '품목코드', a.code_name as '품목명', " +
                    " b.scost as '단가', sum(total) as '소요량', " +
                    " sum(total) * b.scost as '전체가격', b.qom as '현재고', " +
                    " b.qom - sum(total)  as '현재고-소요량', (b.qom - sum(total) ) * b.scost as '부족재고분전체가격' " +
                    " from F_ExpandBOM('" + code + "', " + qty + ") a, t_item b " +
                    " where a.code = b.code " +
                    " and a.level <= " + strLevel + " + 1" +    // 현재 레벨보다 한단계 많게...
                    " and a.code <> '" + code + "'" +           // 조회코드는 생략하고
                    " group by a.code, a.code_name, b.scost, b.qom order by '부족재고'";

                // TODO: 아래 코드 중복 해결해야 함
                clsDBHelper oDb = new clsDBHelper();
                // 거래처 정보를 저장한다.
                oDS = oDb.ExecuteQuery(strQuery, "t_bom_tree");
                oDb = null;
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }

            return oDS;
        }

        /// <summary>
        /// BOM의 특정 노드에 대한 트리 구조를 레벨로 표시한다.
        /// </summary>
        /// <returns></returns>
        public DataSet QueryBOMTree(string code, string qty)
        {
            DataSet oDS = new DataSet();
            try
            {
                // 부모폼목코드(p_code) 에 대한
                // 자식품목코드(c_code), 품목명(im_desc), 품목구분(code_detail_nm), 
                // 필요수량(qty), 공급일수(리드타임) 조회
                string strQuery = "select code as '품목코드', level as '레벨', code_name as '품목명', spec as '스펙', " + 
                    " qty as '단위수량', total as '전체수량' " +
                    " from F_ExpandBOM('" + code + "', " + qty + ")";
                strQuery = "select  Replicate('     |', level - 1) + '     - ' + code_name + '  (' " + 
                    " + code + ', unit=' + cast(qty as varchar(10)) + ', " + 
                    " total=' + cast(total as varchar(10)) + ')' " + 
                    " from F_ExpandBOM('" + code + "', " + qty + ")"; 

                // TODO: 아래 코드 중복 해결해야 함
                clsDBHelper oDb = new clsDBHelper();
                // 거래처 정보를 저장한다.
                oDS = oDb.ExecuteQuery(strQuery, "t_bom_tree");
                oDb = null;
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }

            return oDS;
        }

        /// <summary>
        /// 품목코드를 자품목으로 가지는 모품목코드 목록을 조회한다.
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public DataSet QueryFindParentItem(string code)
        {
            DataSet oDS = new DataSet();
            try
            {
                string strQuery = "select p_code as '품목코드', im_desc as '품목명', spec as '스펙' " +
                    " from F_BOM_FindRoot('" + code + "') root, t_item item " + 
                    " where root.p_code = item.code";

                // TODO: 아래 코드 중복 해결해야 함
                clsDBHelper oDb = new clsDBHelper();
                // 거래처 정보를 저장한다.
                oDS = oDb.ExecuteQuery(strQuery, "t_bom_tree");
                oDb = null;
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }

            return oDS;
        }

        public bool Create(string p_code, string c_code, string qty)
        {
            try
            {
                // c_code 하위 품목코드로 p_code가 존재하면
                // 추가하지 않고 바로 반환한다.
                if(CheckBOMTree(p_code, c_code))
                    return false;

                string strQuery = "insert into t_bom (p_code, c_code, qty) " +
                    " values ('" + p_code + "', '" + c_code + "', " + qty + ")";

                clsDBHelper oDb = new clsDBHelper();
                oDb.ExecuteNonQuery(strQuery);
                oDb = null;

                return true;
            }
            catch(Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }

        public void Remove(string p_code, string c_code)
        {
            try
            {
                string strQuery = "delete from t_bom " +
                    " where p_code = '" + p_code + "'" +
                    " and c_code = '" + c_code + "'"; 

                clsDBHelper oDb = new clsDBHelper();
                oDb.ExecuteNonQuery(strQuery);
                oDb = null;
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }
    }

}
