﻿using System;
using System.Collections.Generic;
using System.Text;
// 추가
using System.Collections;
using System.Data;
using FxLib.Utils;

namespace MyOpenBom.Common
{
    /// <summary>
    /// 카테고리 분류정보를 관리한다.
    /// - 최초 카테고리 마스터코드(t_code_m.code)를 받고
    ///     내부 해시데이터 구조가 있는지 비교해서 있으면 바로 리턴하고
    ///     없으면 DB에서 조회/데이터 저장/해당 데이터를 리턴하도록 한다.
    /// </summary>
    public abstract class CodeManagement
    {
        protected CodeComposite oCComposite;                        // Composite 객체를 내장객체로 사용한다.
        protected DataSet oDS;                                      // DB 조회 결과셋, 프로그램 편의상 추가한다.

        public CodeManagement(string code)
        {
            oCComposite = new CodeComposite("root", "root");        // 코드 composite 객체를 생성한다.

            GetRoot(code);
        }

        /// <summary>
        /// 특정 코드에 대한 루트 값을 조회한다.
        /// 해당 정보는 Code Composite 패턴을 통해서 관리된다.
        /// </summary>
        /// <param name="code"></param>
        public abstract void GetRoot(string code);

        // GetRoot를 통해서 DB 조회된 데이터셋을 반환한다.
        public DataSet GetDataSet()
        {
            return oDS;
        }

        /// <summary>
        /// 특정 노드의 code 값을 조회한다.
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        public string GetNode(string code)
        {
            CodeComponent o = oCComposite.Find(code);
            if (o != null)
                return o.Name();
            else
                return "없음";
        }

        /// <summary>
        /// 카테고리 관리자 객체에서 현재 동일 sibling의 노드에 대한 이름 정보를 반환한다.
        /// 그런데, 과연 이렇게 하는게 좋은지는 좀 더 숙고해야 함
        /// 아래 메소드를 호출해서 콤보박스 내용을 초기화 함
        /// </summary>
        /// <returns></returns>
        public IEnumerable GetNodes()
        {
            foreach (CodeComponent t in oCComposite)
            {
                //yield return t.Name();
                yield return t;
            }
        }
    }

}
