﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
// 추가
using System.Data.SqlClient;
using FxLib.Utils;
//using System.Collections;

// 고객관리: 거래처 정보 추가/수정/삭제/조회
namespace MyOpenBom.CustMan
{
    /// <summary>
    /// 거래처 수정/추가 시 사용되는 폼
    /// </summary>
    public partial class frmCustAdd : Form
    {
        clsNational oNational;      // 국가코드 관리 개체
        clsCust oCust;              // 거래처 개체
        int iFlag;                  // 폼 상태정보(0: 추가모드, 1: 수정모드)

        /// <summary>
        /// 국가 선택정보를 콤보박스에 입력한다.
        /// </summary>
        private void initCust()
        {
            if (oCust == null) return;

            try
            {
                // 국가코드 개체 초기화
                // 객체 생성, 콤보박스 개체에 연결
                oNational = new clsNational();
                cboNational.Items.AddRange(oNational.GetNation());

                cboCustCls.Items.Add("매입거래처");
                cboCustCls.Items.Add("매출거래처");

                // 콤보박스에서는 SelectedIndex나 SelectedItem을 통해서
                // 원하는 값을 선택해 보여줄 수 있다.
                cboCustCls.SelectedIndex = oCust.Cls == "0" ? 0 : 1;                    // 매입 또는 매출거래처 선택
                cboNational.SelectedItem = oNational.FindNationValue(oCust.Nat_no);     // 국가코드 선택

                // 폼상의 컨트롤과 그 속성(Property)를 객체의 속성과 연결시킨다.
                // 컨트롤의 텍스트 속성이 변경되면 객체의 속성도 변경되며,
                // 객체의 속성이 변경되면 컨트롤의 속성도 변경된다.
                // 수정 시 부모 폼에서 전달된 값을 표시한다.
                txtCustIdno.DataBindings.Add("Text", oCust, "Idno");
                txtCustDesc.DataBindings.Add("Text", oCust, "Desc");
                txtCustName.DataBindings.Add("Text", oCust, "Name");
                txtCustPhone.DataBindings.Add("Text", oCust, "Phone");
                txtCustPost.DataBindings.Add("Text", oCust, "Post");
                txtCustAddr.DataBindings.Add("Text", oCust, "Addr");
                txtCustCname.DataBindings.Add("Text", oCust, "Cname");
                txtCustType.DataBindings.Add("Text", oCust, "Type");
                txtCustKind.DataBindings.Add("Text", oCust, "Kind");

            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        /// <summary>
        /// 기본 생성자
        /// </summary>
        public frmCustAdd()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 추가생성자
        /// </summary>
        /// <param name="tCust">고객정보</param>
        public frmCustAdd(clsCust tCust)
        {
            InitializeComponent();

            oCust = tCust;  // 거래처개체 초기화
            // iFlag == 0: 거래처정보 추가모드, 1: 수정모드
            iFlag = oCust.Code == "" ? 0 : 1;

            initCust();     // 폼 필드 초기화
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            
        }

        /// <summary>
        /// 거래처 정보를 DB에 신규 입력하거나 기존 정보를 수정한다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCustAddConfirm_Click(object sender, EventArgs e)
        {
            string strMesg = "입력오류";

            try
            {
                #region 입력값 Validation
                // 입력 필수값 체크
                // List<> 구조를 한번 이용해 봤음
                List<TextBox> oList2 = new List<TextBox>();
                oList2.Add(txtCustIdno);     // 사업자등록번호
                oList2.Add(txtCustDesc);     // 거래처명
                oList2.Add(txtCustName);     // 대표자성명
                oList2.Add(txtCustAddr);     // 거래처 주소

                foreach (TextBox t in oList2)
                {
                    if (t.Text == "")
                    {
                        t.Focus();
                        return;
                    }
                }
                #endregion

                // 폼내의 콤보박스 값 처리: 거래처구분, 국가정보
                oCust.Cls = cboCustCls.SelectedIndex.ToString();
                oCust.Nat_no = oNational.FindNationCode(cboNational.SelectedItem.ToString());

                // 거래처 신규 추가
                if (iFlag == 0)
                {
                    //oCust.Nat_no = "kor";
                    oCust.Create();
                    strMesg = "추가 완료";
                }
                // 거래처 기존 수정
                else if (iFlag == 1)
                {
                    oCust.Update();
                    strMesg = "수정 완료";
                }

                MessageBox.Show(strMesg);
                this.Close();
            }
            // 사용자 정의 예외를 만들경우, SqlException을 처리하는 
            // 특별 로직이 별도로 필요하다.
            catch (SqlException err)    
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                if (err.Number == 2627)
                    MessageBox.Show("중복된 거래처코드가 존재합니다.");
                else
                    MessageBox.Show(MyOpenBom.Properties.Resources.strErr02 + err.Number);
            }
            //catch (clsCustAppException err)
            //{
            //    MessageBox.Show(err.ToString());
            //}
            catch (Exception err)
            {
                //clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        private void btnCustAddCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmCustAdd_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)27)
                this.Close();
        }

        private void frmCustAdd_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                this.Close();
        }
    }
}