﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
// 추가
using System.Data.SqlClient;
using FxLib.Utils;

namespace MyOpenBom.CustMan
{
    /// <summary>
    /// frmCustMain 폼에서 frmCustAdd를 호출 시
    /// 1. 데이터 신규 입력은 frmCustAdd 폼에서 추가 이벤트를 발생해서 frmCustMain에 공지하고
    /// 2. 기존 데이터 수정은 clsCust 객체를 이용해서 frmCustAdd 폼에 데이터를 전달한다.
    /// </summary>
    public partial class frmCustMain : Form
    {
        clsCustMan oCustMan;            // 거래처 관리 개체
        frmCustAdd ofrmCustAdd;         // 거래처 추가 또는 수정 폼 다이알로그
        clsCust oCust;                  // 거래처 개체

        public frmCustMain()
        {
            InitializeComponent();

            // 거래처 관리 객체를 생성한다.
            oCustMan = new clsCustMan();
            oCust = new clsCust();

            oCust.MyChanged += new clsCust.ChangedEventHandler(oCust_MyChanged);
            Load_CustInfo();
        }

        // 거래처개체의 추가/수정/삭제 이벤트를 받으면 데이터그리드뷰 
        // 컨트롤을 갱신한다.
        void oCust_MyChanged(object sender, EventArgs e)
        {
            Load_CustInfo();
        }

        private void btnCustAdd_Click(object sender, EventArgs e)
        {
            try
            {
                ofrmCustAdd = new frmCustAdd(oCust.init());
                ofrmCustAdd.ShowDialog();
            }
            catch(Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show("추가 폼 생성에러발생", err.ToString());
            }
        }

        /// <summary>
        /// 거래처 정보를 DB에서 조회 후 listview 에 추가함
        /// </summary>
        /// <returns>0:성공, 1:실패</returns>
        private int Load_CustInfo()
        {
            try
            {
                dtGridCust.DataSource = oCustMan.Read();
                dtGridCust.DataMember = "t_cust";

                // 특정 컬럼은 사용자에게 보이지 않게 한다.
                dtGridCust.Columns[0].HeaderText = "고유번호";
                dtGridCust.Columns[0].Visible = false;
                dtGridCust.Columns[1].HeaderText = "구분";
                dtGridCust.Columns[1].Visible = false;
                dtGridCust.Columns[2].HeaderText = "사업자등록번호";
                dtGridCust.Columns[2].Width = 100;
                dtGridCust.Columns[3].HeaderText = "거래처명";
                dtGridCust.Columns[3].Width = 150;
                dtGridCust.Columns[4].HeaderText = "대표자성명";
                dtGridCust.Columns[5].HeaderText = "전화번호";
                dtGridCust.Columns[6].HeaderText = "우편번호";
                dtGridCust.Columns[6].Visible = false;
                dtGridCust.Columns[7].HeaderText = "거래처주소";
                dtGridCust.Columns[7].Visible = false;
                dtGridCust.Columns[8].HeaderText = "담당자성명";
                dtGridCust.Columns[9].HeaderText = "업태";
                dtGridCust.Columns[9].Visible = false;
                dtGridCust.Columns[10].HeaderText = "업종";
                dtGridCust.Columns[10].Visible = false;
                dtGridCust.Columns[11].HeaderText = "거래국가";
                dtGridCust.Columns[11].Visible = false;

                return 0;
            }
            catch (SqlException err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                if (err.Number == 53)
                {
                    //MessageBox.Show("이런... DB서버에 연결할 수 없습니다. \n " +
                    //    " config.xml 파일에서 Server 아이피가 \n" +
                    //    "제대로 설정되었는지 확인해주세요.");
                    MessageBox.Show(MyOpenBom.Properties.Resources.strErr01);
                }
                else
                    MessageBox.Show(MyOpenBom.Properties.Resources.strErr02 + err.Number);
            }
            catch (Exception err)
            {
                MessageBox.Show(err.ToString());
            }

            return 1;
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            try
            {
                Load_CustInfo();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.ToString());
            }
        }

        private void dtGridCust_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
        }

        private void btnCustDel_Click(object sender, EventArgs e)
        {
            try
            {
                if (MessageBox.Show(dtGridCust.CurrentRow.Cells[2].Value.ToString() + "를 정말 삭제하시겠습니까?",
                    "품목코드 정보", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    oCust.Delete(dtGridCust.CurrentRow.Cells[0].Value.ToString());
                }
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        private void dtGridCust_CellContentDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            DataGridViewRow row = dtGridCust.CurrentRow;

            oCust.Code = row.Cells[0].Value.ToString();
            oCust.Cls = row.Cells[1].Value.ToString();
            oCust.Idno = row.Cells[2].Value.ToString();
            oCust.Desc = row.Cells[3].Value.ToString();
            oCust.Name = row.Cells[4].Value.ToString();
            oCust.Phone = row.Cells[5].Value.ToString();
            oCust.Post = row.Cells[6].Value.ToString();
            oCust.Addr = row.Cells[7].Value.ToString();
            oCust.Cname = row.Cells[8].Value.ToString();
            oCust.Type = row.Cells[9].Value.ToString();
            oCust.Kind = row.Cells[10].Value.ToString();
            oCust.Nat_no = row.Cells[11].Value.ToString();

            ofrmCustAdd = new frmCustAdd(oCust);
            ofrmCustAdd.ShowDialog();
        }
    }
}