﻿using System;
using System.Collections.Generic;
using System.Text;
// 추가
using System.Data;
using System.Collections;
using System.Data.SqlClient;
using MyOpenBom.Common;
using FxLib.Utils;

namespace MyOpenBom.ItemMan
{
    /// <summary>
    /// 품목정보(코드, 단가 등) 관리 객체
    /// </summary>
    public class clsItem
    {
        Hashtable hsItem;       // 품목데이터
        DataSet dsItem;         // 폼목 데이터셋

        public delegate void ChangedEventHandler(object sender, EventArgs e);
        public event ChangedEventHandler clsItemChanged;

        public clsItem()
        {
            Init();
        }

        public void Init()
        {
            // DataSet 을 생성하고 DataTable을 생성한다.
            if(hsItem == null)
                hsItem = new Hashtable();
            
            hsItem.Clear();

            hsItem.Add("code", "");
            hsItem.Add("cat", "");
            hsItem.Add("im_desc", "");
            hsItem.Add("spec", "");
            hsItem.Add("abc", "");
            hsItem.Add("cust_code", "0");       // 기본고래처, 0
            hsItem.Add("qom", "0");             // 기본재고, 0
            hsItem.Add("ss", "10");
            hsItem.Add("scost", "1000");
            hsItem.Add("pansize", "1");
            hsItem.Add("sday", "7");
        }

        // 객체 내부가 변경될 때 마다 이벤트를 발생시킨다.
        protected virtual void OnclsItemChanged(EventArgs e)
        {
            if (clsItemChanged != null)
                clsItemChanged(this, e);
        }

        public int Count()
        {
            try
            {
                return dsItem.Tables[0].Rows.Count;
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                return 0;
            }
        }

        // 품목정보 데이터셋을 반환한다.
        public DataSet Read(string code)
        {
            dsItem = new DataSet();
            string strQuery;

            try
            {
                if (code.Equals(""))
                    strQuery = "select item.code, code.code_detail_nm, im_desc, " +
                        " spec, abc, cust_code, qom, ss, scost, pansize, sday " +
                        " from t_item item, t_code_d code " +
                        " where item.cat = code.code_detail " +
                        " and code.code = 'C0001' and item.item_use = 'Y' order by item.cat asc, code.code_detail_nm asc";
                else
                    strQuery = "select item.code, code.code_detail_nm, im_desc, " +
                        " spec, abc, cust_code, qom, ss, scost, pansize, sday " +
                        " from t_item item, t_code_d code " +
                        " where item.cat = code.code_detail " +
                        " and code.code = 'C0001' " +
                        " and (item.code like '" + code + "%' or item.im_desc like '" + 
                        code + "%') and item.item_use = 'Y' " +
                        " order by item.cat asc, code.code_detail_nm asc";

                clsDBHelper oDb = new clsDBHelper();
                // 거래처 정보를 저장한다.
                dsItem = oDb.ExecuteQuery(strQuery, "t_item");
                oDb = null;

                return dsItem;
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }

        /// <summary>
        /// 품목정보를 입력한다.
        /// </summary>
        public void Create()
        {
            try
            {
                DataSet dsCreateItem = new DataSet();

                // 입력 쿼리문 작성
                string strQuery = string.Format("insert into t_item (code, cat, im_desc, spec, " +
                    "abc, cust_code, qom, ss, scost, pansize, sday) values " +
                    "('{0}', '{1}', '{2}', '{3}', '{4}', {5}, {6}, {7}, {8}, {9}, " +
                    "{10})", hsItem["code"].ToString(), hsItem["cat"].ToString(), hsItem["im_desc"].ToString(), 
                    hsItem["spec"].ToString(), hsItem["abc"].ToString(), hsItem["cust_code"].ToString(),
                    hsItem["qom"].ToString(), hsItem["ss"].ToString(), hsItem["scost"].ToString(), hsItem["pansize"].ToString(),
                    hsItem["sday"].ToString());

                clsDBHelper oDb = new clsDBHelper();
                oDb.ExecuteNonQuery(strQuery);
                oDb = null;

                OnclsItemChanged(EventArgs.Empty);
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }

        public void Update()
        {
            try
            {
                string strQuery = string.Format("update t_item set cat='{0}', im_desc='{1}', " +
                    " spec='{2}', abc='{3}', cust_code='{4}', qom={5}, ss={6}, scost={7}, pansize={8}, sday={9} where code='{10}'",
                    hsItem["cat"].ToString(), hsItem["im_desc"].ToString(),
                    hsItem["spec"].ToString(), hsItem["abc"].ToString(), hsItem["cust_code"].ToString(),
                    hsItem["qom"].ToString(), hsItem["ss"].ToString(), hsItem["scost"].ToString(), hsItem["pansize"].ToString(),
                    hsItem["sday"].ToString(), hsItem["code"].ToString());

                clsDBHelper oDb = new clsDBHelper();
                oDb.ExecuteNonQuery(strQuery);
                oDb = null;

                OnclsItemChanged(EventArgs.Empty);
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }

        public void Delete(string code)
        {
            try
            {
                // 실제 삭제하지 않는다. 해당 코드의 과거 입출고 이력정보를 보존해야 한다.
                //string strQuery = "delete from t_item where code = '" + code + "'";
                string strQuery = "update t_item set item_use = 'N', del_date = getdate() where code = '" + code + "'";

                clsDBHelper oDb = new clsDBHelper();
                oDb.ExecuteNonQuery(strQuery);
                oDb = null;

                OnclsItemChanged(EventArgs.Empty);
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                throw err;
            }
        }

        #region 속성정의
        /// <summary>
        /// 품목코드
        /// </summary>
        public string Code
        {
            get { return hsItem["code"].ToString(); }
            set { hsItem["code"] = value; }
        }

        /// <summary>
        /// 카테고리
        /// </summary>
        public string Cat
        {
            get { return hsItem["cat"].ToString(); }
            set { hsItem["cat"] = value; }
        }
        /// <summary>
        /// 품목명
        /// </summary>
        public string Im_desc
        {
            get { return hsItem["im_desc"].ToString(); }
            set { hsItem["im_desc"] = value; }
        }

        /// <summary>
        /// 스펙
        /// </summary>
        public string Spec
        {
            get { return hsItem["spec"].ToString(); }
            set { hsItem["spec"] = value; }
        }

        /// <summary>
        /// ABC 구분
        /// </summary>
        public string ABC
        {
            get { return hsItem["abc"].ToString(); }
            set { hsItem["abc"] = value; }
        }

        /// <summary>
        /// 안전재고
        /// </summary>
        public string Ss
        {
            get { return hsItem["ss"].ToString(); }
            set { hsItem["ss"] = value; }
        }

        /// <summary>
        /// 단가
        /// </summary>
        public string scost
        {
            get { return hsItem["scost"].ToString(); }
            set { hsItem["scost"] = value; }
        }

        /// <summary>
        /// 포장규격
        /// </summary>
        public string pansize
        {
            get { return hsItem["pansize"].ToString(); }
            set { hsItem["pansize"] = value; }
        }

        /// <summary>
        /// 공급일수
        /// </summary>
        public string sday
        {
            get { return hsItem["sday"].ToString(); }
            set { hsItem["sday"] = value; }
        }
        #endregion
    }
}
