﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
// 추가
using MyOpenBom.Common;
using System.Data.SqlClient;
using FxLib.Utils;

namespace MyOpenBom.ItemMan
{
    public partial class frmItemAdd : Form
    {
        //DataSet dsItem;     // 품목단위 정보
        clsItem oItem;      // 품목단위 객체
        clsCategoryManagement oCat;
        clsCategoryManagement oABC;

        int iFlag;                  // 폼 상태정보(0: 추가모드, 1: 수정모드)

        public frmItemAdd(clsItem obj)
        {
            InitializeComponent();

            // 객체 생성
            //oItem = new clsItem();
            oItem = obj;
            // iFlag == 0: 품목정보 추가모드, 1: 수정모드
            iFlag = oItem.Code == "" ? 0 : 1;

            // 품목 카테고리 구분 정보 초기화
            InitCategory();
        }

        private void InitCategory()
        {
            string strTmp = "";

            try
            {

                // 품목 구분 관련 카테고리 정보: C0001
                // 규격 구분 카테고리 정보: C0002
                // ABC 구분 카테고리 정보: C0003
                oCat = new clsCategoryManagement("C0001");
                oABC = new clsCategoryManagement("C0003");

                // 품목 구분정보를 콤보박스에 설정한다.
                foreach (CodeComponent oCode in oCat.GetNodes())
                    cboItemCategory.Items.Add(oCode.Name());
                //cboItemCategory.SelectedIndex = 0;
                cboItemCategory.SelectedItem = oItem.Cat;

                // 수정일 경우, 품목에 대한 카테고리 분류와 품목코드는 변경 불가능하게 한다.
                if (iFlag == 1)
                {
                    cboItemCategory.Enabled = false;
                    txtItemCode.Enabled = false;
                    btnAdd.Text = "수정";
                }

                // ABC 구분정보 설정
                foreach (CodeComponent oCode in oABC.GetNodes())
                {
                    cboABC.Items.Add(oCode.Name());
                    // 수정 시 사용자가 선택한 품목코드에 대한 이름값을 조회 후 콤보박스를 설정한다.
                    if (oCode.Code() == oItem.ABC)
                        strTmp = oCode.Name();
                }
                cboABC.SelectedItem = strTmp;

                // 컨트롤 속성 바인딩
                txtItemCode.DataBindings.Add("Text", oItem, "code");            // 품목코드
                txtItemName.DataBindings.Add("Text", oItem, "im_desc");         // 품목명
                txtSpec.DataBindings.Add("Text", oItem, "spec");                // 스펙
                txtSs.DataBindings.Add("Text", oItem, "ss");                    // 안전재고
                txtScost.DataBindings.Add("Text", oItem, "scost");              // 단가
                txtPansize.DataBindings.Add("Text", oItem, "pansize");          // 포장단위
                txtSday.DataBindings.Add("Text", oItem, "sday");                // 공급일수
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }

        }

        private void label2_Click(object sender, EventArgs e)
        {

        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            string strMesg = "입력오류";

            try
            {
                // 필수항목 체크
                #region 입력값 Validation
                // 입력 필수값 체크
                // List<> 구조를 한번 이용해 봤음
                List<TextBox> oList2 = new List<TextBox>();
                oList2.Add(txtItemCode);     // 품목코드
                oList2.Add(txtItemName);     // 품목명

                foreach (TextBox t in oList2)
                {
                    if (t.Text == "")
                    {
                        t.Focus();
                        return;
                    }
                }
                #endregion


                // 컨트롤 속성에 직접 매핑되지 않은 속성들... 예를 들면
                // 콤보박스 선택등은 Create() 메소드 호출 전에 적절한 값으로
                // 변환해서 연결 객체의 속성값을 변경해주어야 한다.
                // 작업: 선택된 이름 => 코드로 변환
                // (카테고리코드, 카테고리명), (Code, Name) (A, 완제품), (B, 상품)
                foreach (CodeComponent oCode in oCat.GetNodes())
                    if (cboItemCategory.SelectedItem.Equals(oCode.Name()))
                    {
                        oItem.Cat = oCode.Code();
                        break;
                    }

                // ABC 구분정보 설정
                foreach (CodeComponent oCode in oABC.GetNodes())
                    if (cboABC.SelectedItem.Equals(oCode.Name()))
                    {
                        oItem.ABC = oCode.Code();
                        break;
                    }
                
                // 거래처 신규 추가
                if (iFlag == 0)
                {
                    // Create() 메소드 호출
                    oItem.Create();
                    strMesg = "추가 완료";
                }
                // 거래처 기존 수정
                else if (iFlag == 1)
                {
                    oItem.Update();
                    strMesg = "수정 완료";
                }

                MessageBox.Show(strMesg);
                this.Close();
            }
            catch (SqlException err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                if (err.Number == 2627)
                    MessageBox.Show("중복된 품목코드가 존재합니다.");
                else
                    MessageBox.Show(MyOpenBom.Properties.Resources.strErr02 + err.Number);
            }
            catch (Exception err)
            {
                clsLog.Write(clsInfo.FATAL, err.ToString());
                MessageBox.Show(err.ToString());
            }
        }

        private void frmItemAdd_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                this.Close();
        }
    }
}
